package com.rf.kjb.user.rest;

import com.alibaba.fastjson.JSONObject;
import com.rf.kjb.base.rest.BaseController;
import com.rf.kjb.scale.util.DateUtil;
import com.rf.kjb.user.dao.model.UserEntry;
import com.rf.kjb.user.service.UserService;
import com.rf.kjb.exception.ErrorCode;
import com.rf.kjb.utils.Constant;
import com.rf.kjb.utils.JWTUtil;
import com.rf.kjb.utils.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

/**
 * @Author:zzf
 * @Date:2022/7/22:17:47
 * @Description:KJB 用户管理
 */
@RestController
@RequestMapping("/v1/user")
@Slf4j
@Api(tags = "用户管理")
public class UserController extends BaseController {

    @Autowired
    private UserService userService;

    /**
     * 注册
     * @param json
     * @return
     */
    @PostMapping("/register")
    @ApiOperation(value = "用户注册",notes = "\"userName 用户名 字符串，identifier：用户编号 字符串，age 年龄 整数数值类型，gender 性别 0女 1男 整数数值类型，password：密码，rePassword：确认密码，profession：职业，role 角色，数值类型，0普通用户 1管理员\"")
    public Result save( @RequestBody String json){
        JSONObject jsonObject =JSONObject.parseObject(json);
        UserEntry userEntry = jsonObject.toJavaObject(UserEntry.class);
        assert StringUtils.isNotBlank(userEntry.getIdentifier()):"编号为空";
        assert StringUtils.isNotBlank(userEntry.getUserName()):"用户名为空";
        assert userEntry.getAge() != 0 :"年龄不合法";
        assert StringUtils.isNotBlank(userEntry.getProfession()):"职业为空";
        assert userEntry.getGender() == 0||userEntry.getGender() == 1 : "性别不合法";
        assert StringUtils.isNotBlank(userEntry.getPassword()) : "密码为空";
        assert StringUtils.isNotBlank(jsonObject.getString("rePassword")) : "确认密码为空";
        if(!userEntry.getPassword().equals(jsonObject.getString("rePassword"))){
            return fail(ErrorCode.PASSWORD_DIFF);
        }
        if(this.userService.findByIdentifier(userEntry.getIdentifier()) != null){
            return  fail(ErrorCode.USER_EXISTS);
        }
        userEntry.setCreateTime(new Date());
        this.userService.save(userEntry);
        return success();
    }

    @PostMapping("/reset")
    @ApiOperation(value = "管理员重置用户密码",notes = "identifier:管理员编号；identifierInit：被重置者编号")
    public Result restPassword(@RequestBody JSONObject jsonObject){
        assert StringUtils.isNotBlank(jsonObject.getString("identifier")):"编号为空";
        assert StringUtils.isNotBlank(jsonObject.getString("identifierInit")):"被重置者编号为空";
        UserEntry userEntry = this.userService.findByIdentifier(jsonObject.getString("identifier"));
        assert userEntry.getRole() == Constant.ROLE_ADMIN : "权限不足";
        userEntry.setUpdateTime(new Date());
        this.userService.resetPasswordByIdentifier(jsonObject.getString("identifierInit"),Constant.DEFAULT_PASSWORD);
        return success();
    }

    /**
     * 登录
     * @param json
     * @return
     */
    @PostMapping("/login")
    @ApiOperation(value = "用户登录",notes = "identifier:用户编号；password：用户密码；role：角色")
    public Result login (@RequestBody String json){
        UserEntry userEntry = JSONObject.parseObject(json).toJavaObject(UserEntry.class);
        UserEntry entry = this.userService.findByIdentifier(userEntry.getIdentifier());
        if(entry == null){
            return fail(ErrorCode.USER_NOT_EXISTS);
        }
        if(!userEntry.getPassword().equals(entry.getPassword())){
            return fail(ErrorCode.LOGIN_ERROR);
        }
        if(userEntry.getRole() != entry.getRole()){
            return fail(ErrorCode.ROLE_ERROR);
        }
        //用户信息验证通过，签发token
        JSONObject resultJson = new JSONObject();
        resultJson.put("token",JWTUtil.getTokenByUserInfo(entry));
        return success(resultJson);
    }


    @PostMapping("/find")
    @ApiOperation(value = "用户信息分页查询",notes = "pageNum:页码；pageSize：每页记录数；userName：用户名；role：角色")
    public Result<Page<UserEntry>> find(@RequestBody String json){
        Page<UserEntry> userEntries =  this.userService.find(json);
        return success(userEntries);
    }

    @PostMapping("/change/password")
    @ApiOperation(value = "重置密码",notes = "identifier:编号 password：密码 newPassword：新密码")
    public Result<ErrorCode> modPass(@RequestBody String json){
        JSONObject jsonObject = JSONObject.parseObject(json);
        UserEntry userEntry = this.userService.findByIdentifierAndPassword( jsonObject.getString("identifier"),jsonObject.getString("password"));
        if(userEntry == null){
            return  fail(ErrorCode.OLD_PWD_ERROR);
        }else {
            userEntry.setPassword(jsonObject.getString("newPassword"));
            userEntry.setUpdateTime(DateUtil.getCurrentDate());
            this.userService.save(userEntry);
            return success(ErrorCode.SUCCESS);
        }
    }
    @ApiOperation("APP验证用户是否存在")
    @GetMapping("{identifier}/checkUser")
    public String checkUser(@PathVariable String identifier) {
        UserEntry userEntry = this.userService.findByIdentifier(identifier);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("userName",userEntry.getUserName());
        jsonObject.put("ID",userEntry.getIdentifier());
        return jsonObject.toJSONString();
    }

    @GetMapping("/hello")
    @ApiOperation("APP扫描服务器")
    public String hello(){
        return "hello";
    }


}
